#!/usr/bin/env python

from Bio import SeqIO


def get_target_seq(args):
    with open(args.target_list, 'r') as fl:
        target_seq_header = fl.read().splitlines()
        target_seq_header = list(set(target_seq_header))
        print(f'Target list: {len(target_seq_header)} sequence(s)')
    target_records = []
    for seq_record in SeqIO.parse(args.input_fasta, "fasta"):
        seq_desc = seq_record.description
        seq_id = seq_record.id
        if seq_desc in target_seq_header:
            target_records.append(seq_record)
            target_seq_header.remove(seq_desc)
        elif seq_id in target_seq_header:
            target_records.append(seq_record)
            target_seq_header.remove(seq_id)
    print(f'No match: {len(target_seq_header)} sequence(s) ')
    # print no match list
    if len(target_seq_header) > 0:
        print('No match list:')
        for h in target_seq_header:
            print(h)
    SeqIO.write(target_records, args.output_fasta, "fasta")



import os

from wlabkit.parse_gbk import get_seq_records, input_type, write_fasta


def get_sl_seq(fasta, l, r):
    """Get seq record objs with specified length.

    Args:
        fasta: A fasta file.
        l: Specifies the minimum length of a sequence. The left edge of the length range and includes the edge value.
        r: Specifies the maximum length of a sequence. The right edge of the length range and includes the edge value.

    Returns:
        A list of specified length seq record objs.
    """

    sl_seq_records = []
    seq_records = get_seq_records(fasta, 'fasta')
    for seq_record in seq_records:
        seqlen = len(seq_record.seq)
        if l <= seqlen <= r:
            sl_seq_records.append(seq_record)
    return sl_seq_records


def exsl_seq(fasta, l, r, merge=False):
    """Extract specified length of sequence.

    Args:
        fasta: A fasta file or a folder containing fasta files.
        merge: If true, all gbk file will merge to a fasta file. Otherwise, a gbk file outputs a fasta file.
        l: Specifies the minimum length of a sequence. The left edge of the length range and includes the edge value.
        r: Specifies the maximum length of a sequence. The right edge of the length range and includes the edge value.

    Returns:
        A list or a dict of specified seq record objs.
    """
    in_type = input_type(fasta)
    if in_type == 'folder':
        file_names = os.listdir(fasta)
        file_dirs = [os.path.join(fasta, name) for name in file_names]
        merged_seq_records = []
        no_merged_seq_records = {}
        for file in file_dirs:
            one_file_seq_records = get_sl_seq(file, l, r)
            if merge:
                merged_seq_records += one_file_seq_records
            else:
                fasta_file_name = os.path.splitext(os.path.basename(file))[0] + '.fna'
                no_merged_seq_records[fasta_file_name] = one_file_seq_records

        if merge:
            return merged_seq_records
        else:
            return no_merged_seq_records

    if in_type == 'file':
        seq_records = get_sl_seq(fasta, l, r)
        return seq_records


def exsl_seq_pipeline(args):
    sl_seq = exsl_seq(args.input, args.left, args.right, args.merge)
    write_fasta(sl_seq, args.output)



import os
from collections import namedtuple
import pandas as pd
from bs4 import BeautifulSoup


class AntismashIndexHtmlParser():
    def __init__(self, index_html):
        self.index_html = index_html

    @property
    def soup(self):
        return BeautifulSoup(open(self.index_html), features="html.parser")

    def get_predicted_core_structure(self):
        """
        Parse index.html and extract predicted core structure(s) information.
        {(CP002995.1, Region 1, NRPS+betalactone): {location: smi}}
        """

        # seq_id_ls, region_num_ls, bgc_type_ls = [], [], []
        # location_start_ls, location_end_ls, smi_ls = [], [], []
        genome_structure_dict = {}

        region_pages = self.soup.select('div[class="page"]')
        for region_page in region_pages:
            # 获取每个region信息
            RegionInfo = namedtuple('RegionInfo', ['seq_id', 'region_num', 'bgc_type'])
            if region_page.attrs['id'] == 'overview':
                continue
            region_description = region_page.find(name='div', attrs={'class': 'heading'}).contents[0]
            region_info = RegionInfo(*region_description.strip().split(' - '))

            # 获取每个region里的结构信息（每个region可能有多个结构信息）
            region_structure_dict = {}
            structures = region_page.select('div[class="structure"]')
            for structure in structures:
                prediction_text = structure.find(name='div', attrs={'class': 'prediction-text'}).contents
                if prediction_text:
                    location = prediction_text[0].strip().rstrip(':').split('location ')[1].split(' - ')
                    location_start, location_end = location
                    smi = structure.select('canvas[class="smiles-canvas"]')[0]['data-smiles']
                    region_structure_dict[(location_start, location_end)] = smi
            if region_structure_dict:
                genome_structure_dict[region_info] = region_structure_dict
        return genome_structure_dict


def get_antismash_structures(args):
    html_ls = []
    if args.html_list:
        with open(args.html_list, 'r') as fl:
            html_ls = fl.read().splitlines()
            html_ls = list(set(html_ls))
    if args.input_html:
        html_ls.append(args.input_html)

    structures_df_ls = []
    for html in html_ls:
        antismash_index_html = AntismashIndexHtmlParser(html)
        structures_dict = antismash_index_html.get_predicted_core_structure()
        seq_id, region_num, bgc_type = [], [], []
        location_start, location_end, smi = [], [], []

        for seq_info, structures in structures_dict.items():
            for location, smiles in structures.items():
                seq_id.append(seq_info.seq_id)
                region_num.append(seq_info.region_num)
                bgc_type.append(seq_info.bgc_type)
                location_start.append(location[0])
                location_end.append(location[1])
                smi.append(smiles)
        structures_df = pd.DataFrame({
            'seq_id': seq_id,
            'region_num': region_num,
            'BGC_type': bgc_type,
            'location start': location_start,
            'location end': location_end,
            'SMILES': smi
        })
        structures_df_ls.append(structures_df)
    pd.concat(structures_df_ls).to_csv(args.output_file, sep='\t', index=False)



#!/usr/bin/env python

from Bio import SeqIO
import os
import webbrowser
import zipfile
import pandas as pd



class Antismash_gbk_parser():
    """parse antiSMASH gbk file and extract something needed"""
    def __init__(self):
        self.type = []

    def extract_BGC_type(self, gbk):
        for seq_record in SeqIO.parse(gbk, "genbank"):
            features = seq_record.features
            for feature in features:
                if feature.type == 'protocluster':
                    product = feature.qualifiers['product']
                    self.type += product
        return product


class Antismash_result_folder():
    """read an antiSMASH result folder and extract something needed"""
    def __init__(self, folder_path, decompression=False):
        """initialize features"""

        "get folder path"
        self.path = folder_path

        "get folder name"
        self.basename = os.path.basename(folder_path)

        "get folder dirname"
        self.dirname = os.path.dirname(folder_path)

        "get BGC region bgk files"
        self.region_gbk = []
        self.region_gbk_dir = []

        "get index.html in folder"
        self.index_html = ''
        self.index_html_dir = ''

        if zipfile.is_zipfile(self.path):
            if decompression:
                self.decompression()

                "get files in folder"
                self.path = os.path.join(self.dirname, self.basename)
                self.file = os.listdir(self.path)

            else:
                raise Exception("Input folder is a ZIP file, Please set argument 'decompression' as True.")

        else:
            "get files in folder"
            self.file = os.listdir(self.path)


        self.get_target_file()

    def decompression(self):
        """decompresse zip file to the current folder with name unchanged"""
        zip_file = zipfile.ZipFile(self.path)
        self.basename = self.basename.rstrip('.zip')
        zip_file.extractall(os.path.join(self.dirname, self.basename))
        zip_file.close()


    def get_target_file(self):
        for f in self.file:
            suffix = '.gbk'
            key_word = 'region'
            if (f.endswith(suffix)) and (key_word in f):
                self.region_gbk.append(f)
                self.region_gbk_dir.append(os.path.join(self.path, f))
            if f == 'index.html':
                self.index_html = f
                self.index_html_dir = os.path.join(self.path, f)

    def open_index_html(self):
        index_html_url = 'file://' + self.index_html_dir
        webbrowser.open(index_html_url)

def find_target_gbk_dir(gbkdir_type_dict, target):
    target_dict = {}
    for k, v in gbkdir_type_dict.items():
        for target_v in target:
            if v == target_v:
                target_dict[k] = v
    return target_dict


def get_target_type_gbk_dir(args):
    try:
        with open(args.target_type) as fl:
            target_type = fl.read().splitlines()
    except:
        target_type = [args.target_type]

    antismash_folder = Antismash_result_folder(args.input_path, decompression=True)
    region_gbk_dir = antismash_folder.region_gbk_dir
    antismash_gbk_parser = Antismash_gbk_parser()
    for dir in region_gbk_dir:
        antismash_gbk_parser.extract_BGC_type(dir)
    gbkdir_type_dict = dict(zip(region_gbk_dir, antismash_gbk_parser.type))
    target_dict = find_target_gbk_dir(gbkdir_type_dict, target_type)
    if args.output_file:
        target_df = pd.DataFrame({
            'gbk_dir': list(target_dict.keys()),
            'type': list(target_dict.values())
        })
        target_df.to_csv(args.output_file, index=False, header=None, sep='\t')
    else:
        for k, v in target_dict.items():
            print(f'{k}\t{v}')



#!/usr/bin/env python

from setuptools import find_packages
from setuptools import setup
import codecs
import os.path

def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")
version=get_version("src/wlabkit/__init__.py")

with open("README.rst", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='wlabkit',
    version=version,
    url='https://github.com/BioGavin/wlab',
    license='GPL',
    author='Zhen-Yi Zhou',
    author_email="gavinchou64@gmail.com",
    description="A toolkit to handle bio-data from WeiBin Lab.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=["Programming Language :: Python :: 3.8",
                 "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
                 "Operating System :: OS Independent"],
    scripts=['scripts/wlabkit'],
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    include_package_data=True,
    python_requires=">=3.8",
    install_requires=[
        "biopython",
        "numpy",
        "pandas",
        "python-dateutil",
        "pytz",
        "six"
    ]
)



import os
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord


def input_type(input):
    """Determine if the input is a file or folder.

    Args:
        input: Path of file or folder

    Returns:
        If input is a file, return "file".
        If input is a folder, return "folder".
    """

    if os.path.isdir(input):
        return "folder"
    elif os.path.isfile(input):
        return "file"
    else:
        raise Exception('Unable to recognize input')


def get_seq_records(gene_file, file_type):
    """Parse a gene file.

    Args:
        gene_file: A file containing genes in fasta or genbank format.
        file_type: String of "genbank" or "fasta"

    Returns:
        A list of seq record objs.
    """

    seq_records = []
    for seq_record in SeqIO.parse(gene_file, file_type):
        seq_records.append(seq_record)
    return seq_records


def gbk2fasta(gbk, merge=False):
    """Convert gbk file to fasta file.

    Args:
        gbk: A gbk file or a folder containing gbk files.
        merge: If true, all gbk file will merge to a fasta file. Otherwise, a gbk file outputs a fasta file.

    Returns:
        A list or a dict of seq record objs.
    """

    in_type = input_type(gbk)
    if in_type == 'folder':
        file_names = os.listdir(gbk)
        file_dirs = [os.path.join(gbk, name) for name in file_names]
        merged_seq_records = []
        no_merged_seq_records = {}
        for file in file_dirs:
            one_file_seq_records = get_seq_records(file, 'genbank')
            if merge:
                merged_seq_records += one_file_seq_records
            else:
                fasta_file_name = os.path.splitext(os.path.basename(file))[0] + '.fna'
                no_merged_seq_records[fasta_file_name] = one_file_seq_records

        if merge:
            return merged_seq_records
        else:
            return no_merged_seq_records

    if in_type == 'file':
        seq_records = get_seq_records(gbk, 'genbank')
        return seq_records


def write_fasta(seq_records, output):
    """Save seq records to file(s).

    Args:
        seq_records: Seq record objs.
        output: A file path if output is a file. A folder path if output is multiple files.

    Returns:
        Saved files.
    """

    if isinstance(seq_records, list):
        SeqIO.write(seq_records, output, "fasta")
    if isinstance(seq_records, dict):
        for fn, seq in seq_records.items():
            SeqIO.write(seq, os.path.join(output, fn), "fasta")


def find_cds(gbk):
    """Find CDS sequences in the gbk file.

    Args:
        gbk: A gbk file.

    Returns:
        A list of cds seq record objs.
    """
    seq_records_dict = {}  # {seq_record.description: (locus_ls, cds_ls)}
    for seq_record in SeqIO.parse(gbk, 'genbank'):
        locus_ls, cds_ls = [], []
        seq_records_dict[seq_record.description] = (locus_ls, cds_ls)
        for seq_feature in seq_record.features:
            if seq_feature.type == 'CDS':
                locus_ls += seq_feature.qualifiers['locus_tag']
                cds_ls += seq_feature.qualifiers['translation']

    # create SeqRecord obj.
    cds_seq_records = []
    for desc, feature_seq_info in seq_records_dict.items():
        for idx in range(len(feature_seq_info[0])):
            locus = feature_seq_info[0][idx]
            cds = feature_seq_info[1][idx]
            cds_seq_record = SeqRecord(Seq(cds), id=locus, description=desc)
            cds_seq_records.append(cds_seq_record)
    return cds_seq_records


def ex_cds(gbk, merge=False):
    """Extract CDS sequences from gbk files.

    Args:
        gbk: A gbk file or a folder containing gbk files.
        merge: If true, all gbk file will merge to a fasta file. Otherwise, a gbk file outputs a fasta file.

    Returns:
        A list or a dict of seq record objs.
    """
    in_type = input_type(gbk)
    if in_type == 'folder':
        file_names = os.listdir(gbk)
        file_dirs = [os.path.join(gbk, name) for name in file_names]
        merged_seq_records = []
        no_merged_seq_records = {}
        for file in file_dirs:
            one_file_cds_records = find_cds(file)
            if merge:
                merged_seq_records += one_file_cds_records
            else:
                fasta_file_name = os.path.splitext(os.path.basename(file))[0] + '.fna'
                no_merged_seq_records[fasta_file_name] = one_file_cds_records

        if merge:
            return merged_seq_records
        else:
            return no_merged_seq_records

    if in_type == 'file':
        cds_records = find_cds(gbk)
        return cds_records


def gbk2fasta_pipeline(args):
    seq = gbk2fasta(args.input, args.merge)
    write_fasta(seq, args.output)


def ex_cds_pipeline(args):
    cds = ex_cds(args.input, args.merge)
    write_fasta(cds, args.output)
